// ======================================================================== //
//	Service.h
// ======================================================================== //
//                                                  [][]      [][]          //
//      imservices                                []    []  []    []        //
//                                                      [][][]              //
//      Copyright:                                    []      []            //
//              (c) 2002-2004 Justin Wood,          []  []  []  []          //
//              Alien Technology                    []  []  []  []          //
//              http://www.proteusx.com/            []          []          //
//                                                  [][][][][][][]          //
//                                                  []    []    []          //
//                                                      []  []              //
//                                                      []  []              //
// ==License Agreement===================================================== //
// This program is free software; you can redistribute it and/or modify it  //
// under the terms of the GNU General Public License as published by the    //
// Free Software Foundation; either version 2 of the License, or (at your   //
// option) any later version.                                               //
//                                                                          //
// This program is distributed in the hope that it will be useful, but      //
// WITHOUT ANY WARRANTY; without even the implied warranty of               //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        //
// General Public License for more details.                                 //
//                                                                          //
// You should have received a copy of the GNU General Public License along  //
// with this program; if not, write to the Free Software Foundation, Inc.,  //
// 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                //
// ======================================================================== //

#import <Foundation/Foundation.h>
#import "CommunicationController.h"
#import "ProteusAccount.h"

#import "ProteusCommunicationController.h"

extern ConnectTypes proxyType;

@interface Service : NSObject
{
	ProteusAccount * _account;
	Contact * _profile;
	NSArray * _contacts;
	NSString * _awayMessage;
	int _status;
	int _encoding;
	//NSMutableSet * informationToGather;
	
    ServiceInterface * interface;

    NSString *name;
    NSString *error;
    NSString *serviceIdentifier;

    ServiceStatus serviceStatus;

    id <ProteusCommunicationController, CommunicationController, NSObject> communicationController;
    BOOL wasError;
}

- (void)removeContact:(Contact *)contact;
- (void)contactUpdate:(Contact *)contact;
- (void)noteContact:(Contact *)contact inGroup:(NSString *)groupName;

- (void)setEncoding:(int)encoding;
- (int)encoding;

- (NSArray *)contacts;
- (Contact *)profile;
- (ProteusAccount *)account;
- (NSString *)awayMessage;
- (NSString *)awayMessageForContact:(Contact *)contact;

- (BOOL)isConnected;
- (BOOL)isDisconnected;

- (NSString *)name;
- (NSString *)serviceIdentifier;
- (NSString *)lastErrorMessage;

- (NSDictionary *)capabilities;

- (void)setServiceInterface:(ServiceInterface *)newInterface;

- (void)setError:(NSString *)errorMessage;
- (BOOL)uncheckedError;

- (void)setConnectionStatus:(ServiceStatus)status;
- (ServiceStatus)connectionStatus;

- (id)init;

- (void)findUser:(id)sender;

- (void)passRequest:(Request *)request;
- (void)passAction:(Action *)action;
- (void)passResponse:(Action *)response;
- (void)updateAccount:(ProteusAccount *)account;

- (Contact *)createContact;
- (Request *)createRequest;
- (Action *)createAction;
- (id)createObjectOfClass:(Class)className;

- (BOOL)isConnected;
- (BOOL)wantsStatusTitles;

- (void)updateUserProfileWithContact:(Contact *)contact;

- (void)setStatus:(ProteusStatus)status withMessage:(NSString *)message;
- (void)setStatus:(ProteusStatus)status withMessage:(NSString *)message withTitle:(NSString *)title;
- (void)getAwayMessageForContact:(Contact *)contact;
- (void)getUserInformation:(Contact *)contact;

- (void)handleRequest:(Request *)request;
- (void)handleAction:(Action *)action;
- (void)handleResponse:(Action *)response;

- (void)userAdded:(Contact *)contact toGroup:(NSString *)to;
- (void)userRemoved:(Contact *)contact;
- (void)userUpdated:(Contact *)contact;

- (void)searchUserFound:(Contact *)contact;
- (void)searchComplete;

- (void)connectAccount:(ProteusAccount *)account withProfileContact:(Contact *)contact withContacts:(NSArray *)contacts;
- (void)disconnect;

- (id <NSObject, CommunicationController>)communicationController;
- (void)displayError:(int)code withAdditionalText:(NSString *)text withRemedy:(NSString *)remedy;
- (void)updateVisibilityLists;
@end
